
/* palette.cc */

#include "palette.h"

#include <QApplication>
#include <QMouseEvent>
#include <QPixmap>
#include <QBitmap>
#include <QPainter>

#if QT_VERSION >= 0x040600
   #define QT_4_6
#endif

/* ---------------------------------------------------------------------- */

void MyPalette::addPage (QString name, MyPalettePage * page)
{
   addTab (page, name);
}

/* ---------------------------------------------------------------------- */

void MyPalettePage::addItem (QString name, MyPaletteItem * item)
{
   if (name != "")
   {
      item->setText (name);
      item->setToolTip (name);
   }
   addWidget (item);
}

void MyPalettePage::addColor (QString name)
{
   addItem (name, new ColorPaletteItem (QColor (name)));
}

void MyPalettePage::addSymbol (QString name, QString group, QString tag)
{
   addItem (name, new SymbolPaletteItem (group, tag));
}

void MyPalettePage::addShape (QString name, shape_func_t func)
{
   addItem (name, new ShapePaletteFunc (func));
}

/* ---------------------------------------------------------------------- */

/*
void MyPaletteItem::mousePressEvent (QMouseEvent * event)
{
   if (event->button() == Qt::LeftButton)
      dragStartPosition = event->pos ();
}

void MyPaletteItem::mouseMoveEvent (QMouseEvent * event)
{
   if (event->button() == Qt::LeftButton ) &&
      (event->pos() - dragStartPosition).manhattanLength() >= QApplication::startDragDistance())
   {
      QDrag * drag = new QDrag (this);
      QMimeData * mimeData = new QMimeData;

      // mimeData->setData (mimeType, data);
      mimeData->setText ("hello");
      drag->setMimeData (mimeData);

      Qt::DropAction dropAction = drag->exec (Qt::CopyAction | Qt::MoveAction);
   }
}
*/

void MyPaletteItem::mousePressEvent (QMouseEvent * event)
{
   if (event->button() == Qt::LeftButton )
   {
      QDrag * drag = new QDrag (this);

      QMimeData * mimeData = new QMimeData;
      setupMimeData (mimeData);
      drag->setMimeData (mimeData);

      setupPixmap (drag);

      Qt::DropAction dropAction = drag->exec (Qt::MoveAction | Qt::CopyAction | Qt::LinkAction);
   }
}

/* ---------------------------------------------------------------------- */

void ColorPaletteItem::setup ()
{
   QPalette p = palette ();
   p.setColor (QPalette::ButtonText, value);
   setPalette (p);

   QPixmap pixmap (12, 12);
   pixmap.fill (Qt::transparent);

   QPainter painter (&pixmap);
   painter.setPen (Qt::NoPen);
   painter.setBrush (QBrush (value));
   painter.drawEllipse (0, 0, 12, 12);
   painter.end ();

   setIcon (pixmap);
}

void ColorPaletteItem::setupMimeData (QMimeData * data)
{
    data->setColorData (value);
}

void ColorPaletteItem::setupPixmap (QDrag * drag)
{
   QPixmap pixmap (12, 12);
   pixmap.fill (Qt::transparent);

   QPainter painter (&pixmap);
   // painter.translate (15, 15);
   // painter.setRenderHint (QPainter::Antialiasing);
   painter.setPen (Qt::NoPen);
   painter.setBrush (QBrush (value));
   painter.drawEllipse (0, 0, 12, 12);
   painter.end ();

   // pixmap.setMask (pixmap.createHeuristicMask());
   drag->setPixmap (pixmap);

   drag->setHotSpot (QPoint (6, 6));
}

/* ---------------------------------------------------------------------- */

void ShapePaletteItem::setupMimeData (QMimeData * data)
{
    data->setData ("application/x-qview-shape", QByteArray ());
}

QGraphicsItem * ellipse ()
{
    QGraphicsEllipseItem * item = new QGraphicsEllipseItem;
    item->setRect (0, 0, 100, 50);
    item->setPen (QColor (Qt::blue));
    item->setBrush (QBrush (Qt::yellow));
    item->setFlag (QGraphicsItem::ItemIsMovable);
    item->setFlag (QGraphicsItem::ItemIsSelectable);
    return item;
};

QGraphicsItem * triangle ()
{
    QGraphicsPolygonItem * item = new QGraphicsPolygonItem;
    QPolygonF polygon;
    polygon << QPointF (25, 0);
    polygon << QPointF (0,  50);
    polygon << QPointF (50, 50);
    item->setPolygon (polygon);

    item->setPen (QColor (Qt::red));
    item->setBrush (QBrush (Qt::green));
    item->setFlag (QGraphicsItem::ItemIsMovable);
    item->setFlag (QGraphicsItem::ItemIsSelectable);
    return item;
};

QGraphicsItem * wave ()
{
    QGraphicsPathItem * item = new QGraphicsPathItem;
    QPainterPath path;
    path.cubicTo (QPointF (50, 0), QPointF (0,  50), QPointF (50, 50));
    item->setPath (path);

    item->setPen (QColor ("brown"));
    QLinearGradient brush (QPointF (0, 0), QPointF (50,  50));
    brush.setColorAt (0, "yellow");
    brush.setColorAt (1, "orange");
    item->setBrush (brush);
    item->setFlag (QGraphicsItem::ItemIsMovable);
    item->setFlag (QGraphicsItem::ItemIsSelectable);
    return item;
};

/* ---------------------------------------------------------------------- */

void SymbolPaletteItem::setupMimeData (QMimeData * data)
{
    data->setData ("application/x-qview-" + group, tag.toAscii ());
}

/* ---------------------------------------------------------------------- */

MyPalette::MyPalette (QWidget* parent)
   : QTabWidget (parent)
{
    MyPalettePage * page1 = new MyPalettePage;
    page1->addSymbol ("area", "area");
    page1->addSymbol ("sub area", "sub-area");

    MyPalettePage * page2 = new MyPalettePage;
    page2->addSymbol ("source", "source");
    page2->addSymbol ("target", "target");

    MyPalettePage * page3 = new MyPalettePage;
    page3->addSymbol ("bool", "field", "bool");
    // page3->addSymbol ("enum", "field", enum");
    page3->addSymbol ("int", "field", "int");
    page3->addSymbol ("long", "field", "long");
    page3->addSymbol ("float", "field", "float");
    page3->addSymbol ("double", "field", "double");
    page3->addSymbol ("string", "field", "string");

    MyPalettePage * page4 = new MyPalettePage;
    page4->addShape ("triangle", triangle);
    page4->addShape ("ellipse", ellipse);
    page4->addShape ("wave", wave);

    MyPalettePage * page5 = new MyPalettePage;
    page5->addColor ("red");
    page5->addColor ("blue");
    page5->addColor ("green");
    page5->addColor ("yellow");
    page5->addColor ("orange");
    page5->addColor ("silver");
    page5->addColor ("gold");
    page5->addColor ("goldenrod");
    page5->addColor ("lime");
    page5->addColor ("lime green");
    page5->addColor ("yellow green");
    page5->addColor ("green yellow");
    page5->addColor ("forest green");
    page5->addColor ("coral");
    page5->addColor ("cornflower blue");
    page5->addColor ("dodger blue");
    page5->addColor ("royal blue");
    page5->addColor ("wheat");
    page5->addColor ("chocolate");
    page5->addColor ("peru");
    page5->addColor ("sienna");
    page5->addColor ("brown");

    this->addPage ("components", page1);
    this->addPage ("ports", page2);
    this->addPage ("fields", page3);
    this->addPage ("shapes", page4);
    this->addPage ("colors", page5);
}

/* ---------------------------------------------------------------------- */

