
/* palette.h */

#ifndef MYPALETTE_H
#define MYPALETTE_H

#include "area.h"

#include <QTabWidget>
#include <QToolBar>
#include <QToolButton>
#include <QDrag>
#include <QMimeData>

/* ---------------------------------------------------------------------- */

class MyPalette;
class MyPalettePage;
class MyPaletteItem;

typedef Area * (* area_func_t) ();
typedef QGraphicsItem * (* shape_func_t) ();

/* ---------------------------------------------------------------------- */

class MyPalette : public QTabWidget
{
   public:
       explicit MyPalette (QWidget * parent = 0);
       // virtual ~ MyPalette ();

       void addPage (QString name, MyPalettePage * page);
};

class MyPalettePage : public QToolBar
{
   public:
      void addItem (QString name, MyPaletteItem * item);
      void addColor (QString name);
      void addSymbol (QString name, QString group, QString tag = "");
      void addShape (QString name, shape_func_t func);
};

class MyPaletteItem : public QToolButton
{
   public:
      void mousePressEvent (QMouseEvent * event);
      // void mouseMoveEvent (QMouseEvent * event);

   protected:
      virtual void setupMimeData (QMimeData * data) { }
      virtual void setupPixmap (QDrag * drag) { }

   private:
      // QPoint dragStartPosition;
};

/* ---------------------------------------------------------------------- */

class ColorPaletteItem : public MyPaletteItem
{
   public:
      ColorPaletteItem () { }
      ColorPaletteItem (QColor c) { setColor (c); }

      QColor color () { return value; }
      void setColor (QColor c) { value = c; setup (); }

   protected:
      virtual void setupMimeData (QMimeData * data);
      virtual void setupPixmap (QDrag * drag);

   private:
      QColor value;

      void setup ();
};

/* ---------------------------------------------------------------------- */

class ShapePaletteItem : public MyPaletteItem
{
   protected:
      virtual void setupMimeData (QMimeData * data);

   public:
      virtual QGraphicsItem * createShape () = 0;
};

class ShapePaletteFunc : public ShapePaletteItem
{
   private:
      shape_func_t func;
   public:
      ShapePaletteFunc  (shape_func_t param) { func = param; }
      virtual QGraphicsItem * createShape () { return func (); }
};

template <class T>
class ShapePaletteTemplate : public ShapePaletteItem
{
   public:
      virtual QGraphicsItem * createShape () { return new T; }
};

/* ---------------------------------------------------------------------- */

class SymbolPaletteItem : public MyPaletteItem
{
   public:
      SymbolPaletteItem (QString g, QString t = "") { group = g; tag = t; }

   protected:
      virtual void setupMimeData (QMimeData * data);

   private:
      QString group;
      QString tag;
};

/* ---------------------------------------------------------------------- */

#endif // MYPALETTE_H
