
/* win.cc */

#include "win.h"

#include "QAction"
#include "QPushButton"
#include "QSplitter"

#include <QApplication>

#if QT_VERSION >= 0x040300
   #define QT_4_3
#endif

#if QT_VERSION >= 0x040500
   #define QT_4_5
#endif

#if QT_VERSION >= 0x040600
   #define QT_4_6
#endif

/* ---------------------------------------------------------------------- */

void example (MyView * view)
{
   Area * a = new Area;
   a->setPen (QPen (Qt::green));
   a->setBrush (Qt::yellow);
   a->setPos (10, 10);
   a->setTitle ("First");
   a->setToolTip ("A");

   ListField * f = new ListField;
   f->setName ("size");
   f->setValue ("small");
   f->list << "small" << "medium" << "large";
   a->addField (f);

   Field * f2 = new Field;
   f2->setName ("color");
   f2->setValue ("blue");
   a->addField (f2);

   Source * a_source = new Source;
   a_source->setPos (90, 0);
   a->addSource (a_source);

   Target * a_target = new Target;
   a->addTarget (a_target);

   Area * b = new Area;
   b->setPen (QPen (Qt::red));
   b->setBrush (QColor ("cornflowerblue"));
   b->setTitle ("Second");
   b->setToolTip ("B");
   b->setPos (320, 10);

   Target * b_target = new Target;
   b->addTarget (b_target);

   Source * b_source = new Source;
   b_source->setPos (90, 0);
   b->addSource (b_source);

   Connection * c = new Connection;
   c->setSource (a_source);
   c->setTarget (b_target);

   #if 0
   Connection * c2 = new Connection;
   c2->setSource (b_source);
   c2->setTarget (a_target);
   #endif

   Area * s = new Area;
   s->setTitle ("orange");
   s->setPen (QPen (Qt::red));
   s->setBrush (QColor ("orange"));
   s->setToolTip ("Subitem");
   s->setRect (10, 10, 80, 20);
   a->addArea (s);

   Area * t = new Area;
   t->setTitle ("orange 2");
   t->setPen (QPen (Qt::red));
   t->setBrush (QColor ("orange"));
   t->setToolTip ("Subitem 2");
   t->setRect (10, 10, 80, 20);
   b->addArea (t);

   #if 0
   Area * u = new Area;
   u->setTitle ("lime");
   u->setPen (QPen (Qt::red));
   u->setBrush (QColor ("lime"));
   u->setToolTip ("Subitem 3");
   u->setRect (10, 40, 80, 20);
   b->addArea (u);

   Area * v = new Area;
   v->setTitle ("green");
   v->setPen (QPen (Qt::red));
   v->setBrush (QColor ("green"));
   v->setToolTip ("Subitem 3");
   v->setRect (10, 70, 80, 20);
   b->addArea (v);
   #endif

   view->addArea (a);
   view->addArea (b);
   view->graphicsView->centerOn (a);
}

/* ---------------------------------------------------------------------- */

MyWin::MyWin (QWidget* parent) :
   QMainWindow (parent),
   mainMenu (NULL),
   statusBar (NULL),
   tab (NULL),
   view (NULL),
   fileMenu (NULL),
   editMenu (NULL),
   viewMenu (NULL),
   show_info (NULL)
{
   setup ();
}

/* ---------------------------------------------------------------------- */

void MyWin::setup ()
{
   mainMenu = new QMenuBar (this);
   // toolbar = new QToolBar (this);
   statusBar = new QStatusBar (this);

   view = new MyView (this);

   /* tab widget */

   tab = new QTabWidget (this);
   #ifdef QT_4_5
      tab->setMovable (true);
   #endif
   // tab->setTabsClosable (true);
   // tab->setUsesScrollButtons (true);
  tab->setTabPosition (QTabWidget::South);

   /* layout */

   QSplitter * splitter = new QSplitter (this);
   splitter->addWidget (view->tree);
   splitter->addWidget (view->graphicsView);
   splitter->addWidget (view->prop);

   QSplitter * center = new QSplitter (this);
   center->setOrientation (Qt::Vertical);
   center->addWidget (view->toolbar);
   center->addWidget (splitter);
   center->addWidget (view->info);

   tab->addTab (center, "view");

   /* menu, toolbar, statusbar */

   this->setMenuBar (mainMenu);
   // this->addToolBar (toolbar);
   this->setCentralWidget (tab);
   this->setStatusBar (statusBar);

   /* actions */

   QAction * quit = new QAction (this);
   quit->setText ("&Quit");
   quit->setShortcut (Qt::CTRL + Qt::Key_Q);
   #ifdef QT_4_6
   quit->setIcon (QIcon::fromTheme ("application-exit"));
   #endif
   QObject::connect (quit, SIGNAL(triggered()), this, SLOT(close()));

   show_info = new QAction (this);
   show_info->setText ("info");
   show_info->setCheckable (true);
   show_info->setChecked (true);
   QObject::connect (show_info, SIGNAL(triggered(bool)), view->info, SLOT(setVisible(bool)));

   /* menu */

   fileMenu = mainMenu->addMenu ("&File");
   fileMenu->addAction (quit);

   viewMenu = mainMenu->addMenu ("&View");
   viewMenu->addAction (show_info);

   editMenu = mainMenu->addMenu ("&Edit");

   /* example */

   example (view);

   Area * r = view->getRoot ();
   r->updateConnections();
   view->tree->setRoot (r);
   view->areaActivated (r->getFirstItem());

}

/* ---------------------------------------------------------------------- */

// #include "win.moc"

/* ---------------------------------------------------------------------- */

extern "C" int main (int argc, char * * argv)
{
    // finch::set_error_handlers ();

    QApplication app (argc, argv);
    app.setApplicationName ("qview");

    MyWin * win = new MyWin ();
    win->show ();

    return app.exec();
}

/* ---------------------------------------------------------------------- */

